@@ -9,6 +9,7 @@ import re
 import shlex
 import subprocess
 import sys
+import tempfile
 import threading
 import time
 import traceback
@@ -17,6 +18,7 @@ from multiprocessing import Process, Queue
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from localstack import config
+from localstack.config import LAMBDA_TRUNCATE_STDOUT, TMP_FOLDER
 from localstack.constants import DEFAULT_LAMBDA_CONTAINER_REGISTRY
 from localstack.runtime.hooks import hook_spec
 from localstack.services.awslambda.lambda_utils import (
@@ -39,6 +41,7 @@ from localstack.utils.aws.dead_letter_queue import (
     sqs_error_to_dead_letter_queue,
 )
 from localstack.utils.cloudwatch.cloudwatch_util import cloudwatched
+from localstack.utils.collections import select_attributes
 from localstack.utils.common import (
     TMP_FILES,
     CaptureOutput,
@@ -63,13 +66,13 @@ from localstack.utils.common import (
     truncate,
     wait_for_port_open,
 )
-from localstack.utils.docker_utils import (
-    DOCKER_CLIENT,
+from localstack.utils.container_utils.container_client import (
     ContainerConfiguration,
     ContainerException,
     DockerContainerStatus,
     PortMappings,
 )
+from localstack.utils.docker_utils import DOCKER_CLIENT
 from localstack.utils.run import FuncThread
 
 # constants
@@ -106,6 +109,9 @@ LAMBDA_CONCURRENCY_LOCK = {}
 # CWD folder of handler code in Lambda containers
 DOCKER_TASK_FOLDER = "/var/task"
 
+# TODO remove once clarification of apigateway contexts is complete. Really bad hack!!
+ALLOWED_IDENTITY_FIELDS = ["cognitoIdentityId", "cognitoIdentityPoolId"]
+
 # Lambda event type
 LambdaEvent = Union[Dict[str, Any], str, bytes]
 
@@ -126,7 +132,7 @@ class InvocationException(Exception):
         self.result = result
 
 
-class LambdaContext(object):
+class LambdaContext:
     DEFAULT_MEMORY_LIMIT = 1536
 
     def __init__(
@@ -139,11 +145,13 @@ class LambdaContext(object):
         self.invoked_function_arn = lambda_function.arn()
         if qualifier:
             self.invoked_function_arn += ":" + qualifier
-        self.cognito_identity = context.get("identity")
+        self.cognito_identity = context.get("identity") and select_attributes(
+            context.get("identity"), ALLOWED_IDENTITY_FIELDS
+        )
         self.aws_request_id = str(uuid.uuid4())
         self.memory_limit_in_mb = lambda_function.memory_size or self.DEFAULT_MEMORY_LIMIT
-        self.log_group_name = "/aws/lambda/%s" % self.function_name
-        self.log_stream_name = "%s/[1]%s" % (timestamp(format="%Y/%m/%d"), short_uid())
+        self.log_group_name = f"/aws/lambda/{self.function_name}"
+        self.log_stream_name = f"{timestamp(format='%Y/%m/%d')}/[1]{short_uid()}"
 
     def get_remaining_time_in_millis(self):
         # TODO implement!
@@ -375,7 +383,7 @@ class LambdaAsyncLocks:
 LAMBDA_ASYNC_LOCKS = LambdaAsyncLocks()
 
 
-class LambdaExecutor(object):
+class LambdaExecutor:
     """Base class for Lambda executors. Subclasses must overwrite the _execute method"""
 
     def __init__(self):
@@ -660,12 +668,19 @@ class LambdaExecutorContainers(LambdaExecutor):
                 log_output += "\n%s" % additional_logs
 
         func_arn = lambda_function and lambda_function.arn()
-        log_lambda_result(func_arn, result, log_output)
+
+        output = OutputLog(result, log_output)
+        LOG.debug(
+            f"Lambda {func_arn} result / log output:"
+            f"\n{output.stdout_formatted()}"
+            f"\n>{output.stderr_formatted()}"
+        )
 
         # store log output - TODO get live logs from `process` above?
         store_lambda_logs(lambda_function, log_output)
 
         if error:
+            output.output_file()
             raise InvocationException(
                 "Lambda process returned with error. Result: %s. Output:\n%s"
                 % (result, log_output),
@@ -718,7 +733,8 @@ class LambdaExecutorContainers(LambdaExecutor):
             environment["AWS_LAMBDA_FUNCTION_NAME"] = context.function_name
             environment["AWS_LAMBDA_FUNCTION_VERSION"] = context.function_version
             environment["AWS_LAMBDA_FUNCTION_INVOKED_ARN"] = context.invoked_function_arn
-            environment["AWS_LAMBDA_COGNITO_IDENTITY"] = json.dumps(context.cognito_identity or {})
+            if context.cognito_identity:
+                environment["AWS_LAMBDA_COGNITO_IDENTITY"] = json.dumps(context.cognito_identity)
             if context.client_context is not None:
                 environment["AWS_LAMBDA_CLIENT_CONTEXT"] = json.dumps(
                     to_str(base64.b64decode(to_bytes(context.client_context)))
@@ -1276,6 +1292,7 @@ class LambdaExecutorLocal(LambdaExecutor):
 
         process = run(cmd, stderr=subprocess.PIPE, outfile=subprocess.PIPE, **kwargs)
         result, log_output = process.communicate()
+
         try:
             result = to_str(result).strip()
         except Exception:
@@ -1298,12 +1315,18 @@ class LambdaExecutorLocal(LambdaExecutor):
                 log_output += "\n%s" % additional_logs
 
         func_arn = lambda_function and lambda_function.arn()
-        log_lambda_result(func_arn, result, log_output)
+        output = OutputLog(result, log_output)
+        LOG.debug(
+            f"Lambda {func_arn} result / log output:"
+            f"\n{output.stdout_formatted()}"
+            f"\n>{output.stderr_formatted()}"
+        )
 
         # store log output - TODO get live logs from `process` above?
         # store_lambda_logs(lambda_function, log_output)
 
         if return_code != 0:
+            output.output_file()
             raise InvocationException(
                 "Lambda process returned error status code: %s. Result: %s. Output:\n%s"
                 % (return_code, result, log_output),
@@ -1326,6 +1349,8 @@ class LambdaExecutorLocal(LambdaExecutor):
         lambda_cwd = lambda_function.cwd
         environment = self._prepare_environment(lambda_function)
 
+        environment["LOCALSTACK_HOSTNAME"] = config.LOCALSTACK_HOSTNAME
+        environment["EDGE_PORT"] = str(config.EDGE_PORT)
         if lambda_function.timeout:
             environment["AWS_LAMBDA_FUNCTION_TIMEOUT"] = str(lambda_function.timeout)
         context = inv_context.context
@@ -1403,7 +1428,14 @@ class LambdaExecutorLocal(LambdaExecutor):
                 error,
                 "".join(traceback.format_tb(error.__traceback__)),
             )
-            raise InvocationException(result, log_output)
+            result = json.dumps(
+                {
+                    "errorType": error.__class__.__name__,
+                    "errorMessage": error.args[0],
+                    "stackTrace": traceback.format_tb(error.__traceback__),
+                }
+            )
+            raise InvocationException(result, log_output=log_output, result=result)
 
         # construct final invocation result
         invocation_result = InvocationResult(result, log_output=log_output)
@@ -1466,13 +1498,9 @@ class LambdaExecutorLocal(LambdaExecutor):
         cmd = [
             "node",
             "-e",
-            'require("%s").%s(%s,%s).then(r => process.stdout.write(JSON.stringify(r)))'
-            % (
-                main_file,
-                function,
-                event_json_string,
-                context_json_string,
-            ),
+            f'const res = require("{main_file}").{function}({event_json_string},{context_json_string}); '
+            f"const log = (rs) => console.log(JSON.stringify(rs)); "
+            "res && res.then ? res.then(r => log(r)) : log(res)",
         ]
         LOG.info(cmd)
         result = self._execute_in_custom_runtime(cmd, lambda_function=lambda_function)
@@ -1571,7 +1599,10 @@ class Util:
         if runtime == "nodejs14.x" and docker_image == DEFAULT_LAMBDA_CONTAINER_REGISTRY:
             # TODO temporary fix until lambci image for nodejs14.x becomes available
             docker_image = "localstack/lambda-js"
-        if runtime == "python3.9" and docker_image == DEFAULT_LAMBDA_CONTAINER_REGISTRY:
+        elif (
+            runtime in ["python3.9", "dotnet6"]
+            and docker_image == DEFAULT_LAMBDA_CONTAINER_REGISTRY
+        ):
             # TODO temporary fix until we support AWS images via https://github.com/localstack/localstack/pull/4734
             docker_image = "mlupin/docker-lambda"
         return "%s:%s" % (docker_image, docker_tag)
@@ -1618,11 +1649,29 @@ class Util:
         return env_vars
 
 
-def log_lambda_result(func_arn, result, log_output):
-    result = to_str(result or "")
-    log_output = truncate(to_str(log_output or ""), max_length=2000)
-    log_formatted = log_output.strip().replace("\n", "\n> ")
-    LOG.debug("Lambda %s result / log output:\n%s\n> %s", func_arn, result.strip(), log_formatted)
+class OutputLog:
+
+    __slots__ = ["_stdout", "_stderr"]
+
+    def __init__(self, stdout, stderr):
+        self._stdout = stdout
+        self._stderr = stderr
+
+    def stderr_formatted(self, truncated_to: int = LAMBDA_TRUNCATE_STDOUT):
+        return truncate(to_str(self._stderr).strip().replace("\n", "\n> "), truncated_to)
+
+    def stdout_formatted(self, truncated_to: int = LAMBDA_TRUNCATE_STDOUT):
+        return truncate(to_str(self._stdout).strip(), truncated_to)
+
+    def output_file(self):
+        try:
+            with tempfile.NamedTemporaryFile(
+                dir=TMP_FOLDER, delete=False, suffix=".log", prefix="lambda_"
+            ) as f:
+                LOG.info(f"writing log to file '{f.name}'")
+                f.write(to_bytes(self._stderr))
+        except Exception as e:
+            LOG.warning("failed to write log to file, error %s", e)
 
 
 # --------------
